# Instumental Variable Models for Discrete Outcomes, Andrew Chesher, November 2nd 2009.

# this file reproduces identified sets for figures 3 and 4 from the paper.  
# Result is saved to "./Data/ivmdoOrdPrb.Rdata".
# Figures 3 and 4 are reproduced by running a script 'figsOrdPrb.R'.
# --------------------------------------------------------------------------------
rm(list=ls())
source("./Libs/oplib.R")		# load library adclib.R
source("./Libs/radarlib.R")		# load library radarlib.R
# --------------------------------------------------------------------------------
# Set inclusion indicator function
OrdProbitCond <- function(zmin,zmax,tauvec,Tm,a,b,suv,svv){
	f <- function(pt){
		res <- TRUE
		for (tau in tauvec) {
			aa <- boundsz(zmin,zmax,tau,Tm,a,b,suv,svv, pt)
			if(aa$ub$objective <tau || aa$lb$objective >tau){res <- FALSE;break } 
		}
	return(res)
	}
}
# --------------------------------------------------------------------------------

zrange <- c(-1,1)	# range of instrument
tauvec <- seq(0.005,0.995,length=25)

# --- Tm 5 classes

tau <- seq(0,1,len=6)		# equally spaced quantiles
Tm5 <- qnorm(tau[2:5])		# 4 thresholds

# set inclusion indicator functions for 5 classes
cf5w <- OrdProbitCond(zmin=-1,zmax=1,tauvec=tauvec,Tm=Tm5,a=c(0,1),b=c(0,1),suv=0.6,svv=1)	# weak instrument	
cf5s <- OrdProbitCond(zmin=-1,zmax=1,tauvec=tauvec,Tm=Tm5,a=c(0,1),b=c(0,2),suv=0.6,svv=1)	# strong instrument

# search for the identified sets
res5w <- radar(initpt=c(0,1),confun=cf5w,psi=pi/10,eps=0.01,step=.1,psistep=0.05*pi,ctrl=TRUE)	
res5s <- radar(initpt=c(0,1),confun=cf5s,psi=pi/10,eps=0.01,step=.1,psistep=0.05*pi,ctrl=TRUE)	


# --- Tm - 11 classes

tau <- seq(0,1,len=12)
Tm11 <- qnorm(tau[2:11])

# set inclusion indicator functions for 11 classes
cf11w <- OrdProbitCond(zmin=-1,zmax=1,tauvec=tauvec,Tm=Tm11,a=c(0,1),b=c(0,1),suv=0.6,svv=1)
cf11s <- OrdProbitCond(zmin=-1,zmax=1,tauvec=tauvec,Tm=Tm11,a=c(0,1),b=c(0,2),suv=0.6,svv=1)

# search for the identified sets
res11w <- radar(initpt=c(0,1),confun=cf11w,psi=pi/10,eps=0.01,step=.1,psistep=0.05*pi,ctrl=TRUE)	
res11s <- radar(initpt=c(0,1),confun=cf11s,psi=pi/10,eps=0.01,step=.1,psistep=0.05*pi,ctrl=TRUE)	

save(res5s,res5w,res11s,res11w,file="./Data/ivmdoOrdPrb.Rdata")
	
# --- end of file	
